import {
  computed,
  defineComponent,
  nextTick,
  onMounted,
  reactive,
  ref
} from 'vue'
import umiRequest from 'umi-request'
import { useRoute, useRouter } from 'vue-router'
import request from '@/helpers/request'
import ColHeader from '@/components/col-header'
import { postMessage, promisefiyPostMessage } from '@/helpers/native-message'
import {
  Button,
  Cell,
  Dialog,
  Icon,
  Image,
  Popup,
  Sticky,
  Tag,
  Toast
} from 'vant'
import styles from './index.module.less'
// import Item from '../list/item'
import { useRect } from '@vant/use'
import { Vue3Lottie } from 'vue3-lottie'
import { getRandomKey } from '@/views/music/music'
import { getOssUploadUrl, state } from '@/state'
import { useEventTracking } from '@/helpers/hooks'
import ColSticky from '@/components/col-sticky'
import { browser, moneyFormat } from '@/helpers/utils'
import { orderStatus } from '@/views/order-detail/orderStatus'
import iconAlbum from '@/views/music/component/images/icon_album.png'
import iconDownload from '@/views/music/music-detail/images/icon_download.png'
import AstronautJSON from '@/views/music/music-detail/animate/bigLoad.json'
import ColShare from '@/components/col-share'
import iconCollect from '@/views/music/music-detail/images/icon_collect.png'
import iconCollectActive from '@/views/music/music-detail/images/icon_collect_active.png'
import iconListen from '@/views/music/music-detail/images/icon_listen.png'
import iconTeacher from '@common/images/icon_teacher.png'
import {
  addMusicTitle,
  addWatermark,
  convasToImg,
  imgToCanvas
} from '@/views/music/music-detail/imageFunction'
import Plyr from 'plyr'
import 'plyr/dist/plyr.css'

import icon_exquisite from '@/views/music/component/images/icon_exquisite.png'
import icon_album_active from '@/views/music/component/images/icon_album_active.png'
import wx_bg from '../images/wx_bg.png'
import { initJumpNativePage, shareCall } from '../share'
import qs from 'query-string'

export default defineComponent({
  name: 'MusicDetail',
  setup() {
    localStorage.setItem('behaviorId', getRandomKey())
    const route = useRoute()
    const loading = ref(false)
    const isError = ref(false)
    const headers = ref(null)
    const footers = ref(null)
    const heightInfo = ref<any>('0')
    const musicDetail = ref<any>(null)
    const showImg = ref<string>('')
    const accompanyUrl = ref<string>('')
    const wxStatus = ref<boolean>(false)

    const tmpUrl = `${location.origin}/student/#/music-detail?${qs.stringify(
      route.query
    )}`
    const jumpUrl = ref<string>(tmpUrl)

    const colors: any = {
      FREE: {
        color: '#01B84F',
        text: '免费'
      },
      VIP: {
        color: '#CD863E',
        text: '会员'
      },
      CHARGE: {
        color: '#3591CE',
        text: '点播'
      }
    }

    const FetchList = async (id?: any) => {
      if (loading.value) {
        return
      }
      loading.value = true
      isError.value = false
      try {
        const search = route.query
        const res = await request.post(`/open/musicShareProfit`, {
          prefix: '/api-teacher',
          requestType: 'json',
          data: {
            bizId: search.id,
            userId: search.recomUserId
          }
        })
        musicDetail.value = res.data.musicSheet
        showImg.value = musicDetail.value?.musicImg || ''

        if (!showImg.value) {
          setAccompanyUrl()
          window.addEventListener(
            'message',
            async e => {
              // 给图片设置背景色
              const tempCanvas = await imgToCanvas(e.data)
              const img = convasToImg(tempCanvas)
              //  开始上传图片
              uploadFunction(img)
            },
            false
          )
        }
      } catch (error) {
        isError.value = true
      }
      loading.value = false
    }
    const base64ToBlob = data => {
      const arr = data.split(','),
        mime = arr[0].match(/:(.*?);/)[1]

      const bstr = atob(arr[1])
      let n = bstr.length
      const u8arr = new Uint8Array(n)

      while (n--) {
        u8arr[n] = bstr.charCodeAt(n)
      }
      return new Blob([u8arr], { type: mime })
    }
    const uploadFunction = async file => {
      try {
        const formData = new FormData()
        const fileName =
          new Date().getTime() +
          musicDetail.value?.musicSheetName.replaceAll(' ', '_') +
          '.png'
        const keyTime = new Date().getTime() + fileName
        const obj = {
          filename: fileName,
          bucketName: 'cloud-coach',
          postData: {
            filename: fileName,
            acl: 'public-read',
            key: keyTime,
            unknowValueField: []
          }
        }

        const res = await request.post(state.platformApi + '/getUploadSign', {
          data: obj
        })

        Toast.loading({
          message: '加载中...',
          forbidClick: true,
          loadingType: 'spinner',
          duration: 0
        })

        const dataObj = {
          policy: res.data.policy,
          signature: res.data.signature,
          key: keyTime,
          KSSAccessKeyId: res.data.kssAccessKeyId,
          acl: 'public-read',
          name: fileName
        }
        for (const key in dataObj) {
          formData.append(key, dataObj[key])
        }
        const files = base64ToBlob(file)

        formData.append('file', files, fileName)
        const ossUploadUrl = getOssUploadUrl('cloud-coach')
        await umiRequest(ossUploadUrl, {
          method: 'POST',
          data: formData
        })
        Toast.clear()
        const imgurl = getOssUploadUrl('cloud-coach') + keyTime

        await request.post(state.platformApi + '/open/music/sheet/img', {
          data: { musicSheetId: musicDetail.value.id, musicImg: imgurl }
        })
        showImg.value = imgurl
      } catch (e) {
        console.log(e)
      }
    }

    const setAccompanyUrl = () => {
      const url = 'http://dev.colexiu.com'
      const music = musicDetail.value
      let subjectId = ''
      if (music.background && music.background.length > 0) {
        subjectId = music.background[0].id
      }
      accompanyUrl.value =
        url +
        `/accompany/colxiu-website.html?id=${music.id}&part-index=${subjectId}`
    }
    const player = ref<any>(null)
    const audio = ref<any>(null)
    const freeRate = ref<any>(0)
    const initAudio = async () => {
      const config = await request.get(
        state.platformApi + '/sysConfig/queryByParamNameList',
        {
          params: {
            paramNames: 'music_sheet_free_rate'
          }
        }
      )
      freeRate.value = config.data[0]?.paramValue || 0

      const controls = [
        // 'play-large',
        'play',
        'progress',
        // 'captions',
        // 'fullscreen',
        'duration'
      ]
      player.value = new Plyr(audio.value, {
        controls: controls
      })

      player.value.on('timeupdate', () => {
        // 允许播放时间
        const players = player.value
        const playTime = (players.duration * freeRate.value) / 100 || 0
        // 时间，是否购买，是否免费
        if (
          players.currentTime >= playTime &&
          musicDetail.value?.orderStatus !== 'PAID' &&
          !paymentType.value.includes('FREE')
        ) {
          // players.stop()
          players.pause()
        }
      })
    }
    onMounted(async () => {
      initJumpNativePage(jumpUrl.value)
      await FetchList()
      const { height } = useRect(headers as any)
      const footer = useRect(footers as any)
      heightInfo.value = height + footer.height

      // 初始化音频
      if (musicDetail.value?.audioFileUrl) {
        initAudio()
      }
    })

    const paymentType = computed(() => {
      let paymentType = musicDetail.value?.paymentType
      if (typeof paymentType === 'string') {
        paymentType = paymentType.split(',')
        return paymentType
      }
      return []
    })

    const onShare = () => {
      console.log(browser().weixin)
      if (browser().weixin) {
        wxStatus.value = true
        return
      }
      // 尝试拉起app
      shareCall(jumpUrl.value)
      // 不管有没有拉起app则都跳转到下载app
      setTimeout(() => {
        window.location.href = location.origin + '/student/#/download'
      }, 3000)
    }

    return () => {
      return (
        <div class={styles.detail}>
          <Sticky position="top">
            <div ref={headers}>
              <ColHeader
                background="transparent"
                border={false}
                isFixed={false}
                color="#fff"
                backIconColor="white"
              />
            </div>
          </Sticky>

          <img class={styles.bgImg} src={musicDetail.value?.titleImg} />
          <div class={styles.bgContent}></div>
          <div
            class={styles.musicContainer}
            style={{
              marginTop: '16px',
              height: `calc(100vh - ${heightInfo.value + 16 + 'px'})`
            }}
          >
            <Cell
              border={false}
              center
              class={styles.musicInfo}
              v-slots={{
                icon: () => (
                  <Image
                    class={styles.pImg}
                    src={musicDetail.value?.titleImg}
                  />
                ),
                title: () => (
                  <div class={styles.info}>
                    <h4 class="van-ellipsis">
                      {musicDetail.value?.musicSheetName}
                    </h4>
                    <p
                      style={{
                        display: 'flex'
                      }}
                    >
                      {paymentType.value.map(tag => (
                        <Tag
                          style={{ color: colors[tag].color }}
                          class={styles.tag}
                          type="success"
                          plain
                        >
                          {colors[tag].text}
                        </Tag>
                      ))}
                      {musicDetail.value?.exquisiteFlag === 1 && (
                        <Image
                          class={styles.exquisiteFlag}
                          src={icon_exquisite}
                        />
                      )}

                      {musicDetail.value?.albumNums > 0 && (
                        <Image
                          class={styles.songAlbum}
                          src={icon_album_active}
                        />
                      )}
                      <span style={{ paddingTop: '2px', paddingLeft: '6px' }}>
                        {musicDetail.value?.composer}
                      </span>
                    </p>
                  </div>
                ),
                value: () => (
                  <span class={styles.download} onClick={() => onShare()}>
                    <img src={iconDownload} />
                    下载曲谱
                  </span>
                )
              }}
            />
            <div class={styles.musicContent}>
              <iframe
                id="containerPrint"
                ref="print"
                style="width: 100%;page-break-after:always; height: 0"
                src={accompanyUrl.value}
              />
              <p class={styles.musicTitle}>
                {musicDetail.value?.musicSheetName}
              </p>
              {showImg.value ? (
                <img src={showImg.value} alt="" class={styles.musicImg} />
              ) : (
                <>
                  <Vue3Lottie
                    animationData={AstronautJSON}
                    class={styles.finch}
                  ></Vue3Lottie>
                  <p class={styles.finchLoad}>加载中...</p>
                </>
              )}

              <div class={styles.videoOperation}>
                {musicDetail.value?.audioFileUrl && (
                  <>
                    {(paymentType.value.includes('CHARGE') ||
                      paymentType.value.includes('VIP')) &&
                      musicDetail.value?.orderStatus !== 'PAID' && (
                        <div class={[styles.audition]}>
                          <img src={iconListen} />
                          <span>每首曲目可试听{freeRate.value}%</span>
                        </div>
                      )}

                    <div class={[styles.audio, styles.collectCell]}>
                      <audio id="player" controls ref={audio}>
                        <source
                          src={musicDetail.value?.audioFileUrl}
                          type="audio/mp3"
                        />
                      </audio>
                    </div>
                  </>
                )}

                <div class={[styles.collect, styles.collectCell]}>
                  <div class={[styles.userInfo]}>
                    <img src={musicDetail.value?.userAvatar || iconTeacher} />
                    <span>{musicDetail.value?.userName}</span>
                  </div>

                  <div
                    class={[styles.collectSection]}
                    onClick={() => onShare()}
                  >
                    <span>{musicDetail.value?.favoriteCount}人收藏</span>
                    <img
                      src={
                        musicDetail.value?.favorite
                          ? iconCollectActive
                          : iconCollect
                      }
                    />
                  </div>
                </div>
              </div>
            </div>

            <div
              class={[styles.lookAlbum, styles.collectCell]}
              onClick={() => {
                onShare()
              }}
            >
              <div>
                <img src={iconAlbum} />
                <span>进入曲目所在专辑列表</span>
              </div>
              <Icon name="arrow" size={16} color="#666" />
            </div>
          </div>

          {musicDetail.value?.id && (
            <ColSticky position="bottom" background="white">
              <div ref={footers}>
                <Button
                  round
                  block
                  type="primary"
                  color="linear-gradient(180deg, #59E5D5 0%, #2DC7AA 100%)"
                  onClick={() => onShare()}
                >
                  下载酷乐秀进入详情
                </Button>
              </div>
            </ColSticky>
          )}

          {wxStatus.value && (
            <div
              class={styles.wxpopup}
              onClick={() => {
                wxStatus.value = false
              }}
            >
              <img src={wx_bg} alt="" />
            </div>
          )}
        </div>
      )
    }
  }
})
